#ifndef CFlatDisplays_H
#define CFlatDisplays_H

#include <xmdv/CDisplay.h>
#include <xmdv/CFlatDisplayInformation.h>
#include <xmdv/brush.h>


class CFlatDisplay : public CDisplay
{
public:
	CFlatDisplayInformation* flat_info;

    ARR<double> all_data;
    ARR<double> temp;
    ARR<double> data;
    ARR<double> max;
    ARR<double> min;
    bool show_dimcluster_band;
    bool show_two_axes;
    bool show_wide_axes;

public:
	CFlatDisplay(CFlatDisplayInformation* flat_info);
	CFlatDisplay() {}
	~CFlatDisplay();
	
    //draw the canvas
    void Draw();
	void InputKeyPress(int key);
	void InputKeyRelease(int key);

    //map all_data to data, then draw data. The distortion in data space is done here.
    virtual void DrawSingleData(ARR<double> data, int datano = 0);
    virtual void DrawSingleDataWithDimClusterExtends(ARR<double> mean, ARR<double> max, ARR<double> min, int datano);
    virtual void DrawSingleDataWithTwoAxes(ARR<double> mean, ARR<double> max, ARR<double> min, int datano, bool unbrushed);

//protected:
    //set parameters used for the drawing
    //rewrite when necessary
    virtual void SetParameters();

    //draw axes, or grid
    //rewrite when necessary
    virtual void DrawAxes();

    //set color and style for brushes
    //rewrite when necessary
    virtual void SetBrushesColorAndStyle();

    //draw brushes
    virtual void DrawBrushes();

    //set color and style for unbrushed data
    //rewrite when necessary
    virtual void SetUnbrushedColorAndStyle();

    //draw unbrushed data
    void DrawUnbrushedRegion();

    //set color and style for brushed data
    //rewrite when necessary
    virtual void SetBrushedColorAndStyle();

    //draw brushed data
    void DrawBrushedRegion();

    //set color and style for average of brushes
    //rewrite when necessary
    virtual void SetAverageColorAndStyle();

    //draw average of brushes if required
    void DrawAverage();

    //free space used for drawing
    //rewrite when necessary
    virtual void FreeSpace();

    //draw other things such as lines links star glyphs
    //rewrite when necessary
    virtual void DrawOthers();
    
    //map all_data to data, The distortion in data space is done here.
//    void MapData(ARR<double> all_data);

    //draw a single data
    //need to rewrite
//    virtual void DrawSingleData();

    //draw a single brush
    //need to rewrite
    virtual void DrawSingleBrush(Brush *brush);

};



#endif

