#include <MDS_misc.h>
//#include <gu_link.h>

class XString 
{
public:
	char ch[256];

	XString();
	char* operator =(char *s);
	char* operator =(const XString& xs);
	friend istream& operator >> (istream& is, XString& xs);
	friend ostream& operator << (ostream& os, const XString& xs);
};


class Min_Max_Card
{
public:
	double minimum, maximum;
	int card;

	Min_Max_Card();
	friend istream& operator >> (istream& is, Min_Max_Card& mmc);
	friend ostream& operator << (ostream& os, const Min_Max_Card& mmc );
};


class OKC_data
{
public:
	int dim_num, rec_num;
	IP_Vector<XString> dim_name;
	IP_Vector<Min_Max_Card> min_max_card;
	MDS_Matrix data;

	OKC_data ();
	void create_okc_data(const IP_Matrix<double>& data_mtx, char* dimname);
	OKC_data combine(OKC_data& other);
	friend istream& operator >> (istream& is, OKC_data&);
	friend ostream& operator << (ostream& os, const OKC_data&);
};



inline XString::XString()
{;}

inline char* XString:: operator =(char *s) 
{
	strcpy(ch, s);
	return ch;
}

inline char* XString:: operator =(const XString& xs)
{
	strcpy(ch,xs.ch);
	return ch;
}

istream& operator >> (istream& is, XString& xs) { return (is>>xs.ch);}
ostream& operator << (ostream& os, const XString& xs) { return (os<<xs.ch);}


inline Min_Max_Card::Min_Max_Card():
	minimum (0),
	maximum (0),
	card (0)
	{}

istream& operator >> (istream& is, Min_Max_Card& mmc)
{
	return (is>>mmc.minimum>>mmc.maximum>>mmc.card);
}

ostream& operator << (ostream& os, const Min_Max_Card& mmc )
{
	char tbl='\t';
	return (os<<mmc.minimum <<tbl<<mmc.maximum <<tbl<<mmc.card);
}


inline OKC_data::OKC_data ():
	dim_num (0), 
	rec_num (0),
	dim_name(),
	min_max_card(),
	data ()
	{}


inline void OKC_data::create_okc_data(const IP_Matrix<double>& data_mtx, char* dimname)
{
	dim_num=data_mtx.len1;
	rec_num=data_mtx.len0;
	dim_name.setSize(dim_num);
	min_max_card.setSize(dim_num);
	data.setSize(rec_num, dim_num);

	if(strlen(dimname)!=0) {
		for (int i=0; i<dim_num; i++) {
			char ch[128];
			strcpy (ch, dimname);
			char buff[8];
			strcat(ch,itoa(i, buff, 10));
			strcpy(dim_name.element(i).ch, ch);
		}
	}
	else {
		for (int i=0; i<dim_num; i++) {
			char ch[128];
			strcpy (ch, "dim");
			char buff[8];
			strcat(ch,itoa(i, buff, 10));
			strcpy(dim_name(i).ch, ch);
		}
	}

	for (int i=0; i<dim_num; i++) {
		min_max_card(i).minimum=data_mtx.col(i).min();
		min_max_card(i).maximum=data_mtx.col(i).max();
		min_max_card(i).card=1;
	}

	data=data_mtx;
}


inline OKC_data OKC_data::combine(OKC_data& other) 
{
	OKC_data result;

	result.dim_num=dim_num+other.dim_num;
	result.rec_num =rec_num;

	//result.dim_name.setSize(dim_num);
	//result.min_max_card .setSize(dim_num);
	result.dim_name=dim_name.right_merge(other.dim_name);
	result.min_max_card =min_max_card.right_merge(other.min_max_card );

	result.data=right_merge(data, other.data);

	return result;
}


istream& operator >> (istream& is, OKC_data& okc)
{
	is>>okc.dim_num>>okc.rec_num;

	okc.dim_name.setSize(okc.dim_num);
	okc.min_max_card.setSize(okc.dim_num);
	is>>okc.dim_name;
	is>>okc.min_max_card;

	okc.data.setSize(okc.rec_num, okc.dim_num);
	is>>okc.data;

	return is;
}

ostream& operator << (ostream& os, const OKC_data& okc)
{
	char tbl='\t';
	os<<okc.dim_num<<tbl<<okc.rec_num <<endl;
	for (int i=0; i<okc.dim_name.len; i++) os<<okc.dim_name(i)<<endl;
	for (i=0; i<okc.min_max_card.len; i++) os<<okc.min_max_card(i)<<endl;
	os<<okc.data;

	return os;
}


	



